library(ROracle)
mdrv <- dbDriver('Oracle')
mconn <- dbConnect(mdrv, 'trinity', 'trinity')

# Get attribute names for a given relational table in oracle DB.
#
# Args: 
#  dataset.name: Name of the relational table
# Returns:
#  Attribute names in the form of a list.

bGetAttrnames <- function(dataset.name) {
	# Create a query string
	qString <- paste("select column_name from USER_TAB_COLUMNS where table_name = \'", toupper(dataset.name), "\'", sep = "")
	res <- dbGetQuery(mconn, qString)
	return(res[[1]])
}

bInitFeatureLengths <- function(t,n,y) {
	library(ROracle)
	#str <- paste(obj$numFeatures, mdb$Datasets$OrigNumFeatures, sep=",")
	str <- paste(t,n,y,sep=",")
	res <- system.time(dbGetQuery(mconn, paste("select oreT.initFeatureLengths(\'", str, "\')
from dual", sep="" )))
	return(res)	
}
bInitDecodeIndices <- function(str) {
	library(ROracle)
	res <- system.time(dbGetQuery(mconn, paste("select oreT.initDecodeIndices(\'",str, "\') from
dual", sep="")))
	return(res)	
}

bInitDecodeIndicesMap <- function(str) {
	library(ROracle)
	res <- system.time(dbGetQuery(mconn, paste("select oreT.initDecodeIndicesMap(\'", str, "\')
from dual", sep="")))
	return(res)
}
bSetYIdx <- function(str) {
	library(ROracle)
	res <- system.time(dbGetQuery(mconn, paste("select oreT.setYIdx(\'", str, "\') from dual",
sep="")))
	return(res)
}
bInitNewFeatures <- function(str)
{
	library(ROracle)
	res <- system.time(dbGetQuery(mconn, paste("select oreT.initNewFeatures(\'",str, "\') from
dual", sep="")))
	return(res)	
}
bIterate <- function(str) {
	library(ROracle)
	t <- system.time((dbGetQuery(mconn, paste("select count(oreT.iterate(rdata)) from ", str ,sep=""))), gcFirst=TRUE)
	return(t)
}
bTblIterate <- function(str='ColumbusFordSmall') {
	library(ROracle)
	#SELECT * FROM TABLE(StockPivot(CURSOR(SELECT * FROM StockTable)));
	t <- system.time((dbGetQuery(mconn, paste("SELECT count(*) FROM TABLE(StockPivot(CURSOR(SELECT * FROM ",str ," )))", sep = ""))), gcFirst=TRUE)
	return(t)
}
bCreateObj <- function(str) {
	library(ROracle)
	res <- dbGetQuery(mconn, paste("select oreT.createObj(\'",str,"\') from dual", sep=""))
	return(res) 
}
bDeleteObj <- function(str) {
	library(ROracle)
	res <- dbGetQuery(mconn, paste("select oreT.deleteObj(\'",str,"\') from dual", sep=""))
	return(res) 
}
bInitialize <- function(str) {
	library(ROracle)
	res <- dbGetQuery(mconn, paste("select oreT.initialize(\'", str, "\') from dual", sep=""))
	return(res)
}
bTerminate <- function() {
	library(ROracle)
	res <- dbGetQuery(mconn, "select oreT.terminate() from dual")
	return(res)
}
bResult <- function(str) {
	library(ROracle)
	res <- dbGetQuery(mconn, paste("select oreT.result(\'", str, "\') from dual", sep=""))
	return(res)
}
bNullUDF <- function(str) {
	library(ROracle)
	#print(str)
	t <- system.time(dbGetQuery(mconn, paste("select count(oreT.nullUDF(rdata)) from ", str, " ", sep="")))
	return(t)
}
bNoDataFn <- function(str) {
	library(ROracle)
	t <- system.time(dbGetQuery(mconn, paste("select count(oreT.noDataFunction()) from ", str, " ", sep="")))
	return(t)
}
bNullAndParse <- function(str) {
	library(ROracle)
	t <- system.time(dbGetQuery(mconn, paste("select count(oreT.nullAndParse(rdata)) from ", str,sep="")))
	return(t)
}
bInitObjectList <- function(str) {
	library(ROracle)
	res <- system.time(dbGetQuery(mconn, paste("select oreT.initObjectList(\'",str, "\') from
dual", sep="")))
	return(res)
}

#bMaterialize <- function(tblName, parent) {
#	library(ROracle)
#	s =  paste("drop table " , tblName," ;", sep="")	
#	#print(s)
#	t <- dbGetQuery(mconn, paste("drop table " , tblName, sep=""))
#	#print(paste("calling materialize"))
#	t <- system.time(dbGetQuery(mconn, paste("create table ", tblName," tablespace bigtbs as select oreT.materialize(rdata) as rdata from ", parent," ", sep="")))
#	return(t)	
#}
bSelectMaterialize = function(inpTable, featureNames, outTable, predicate)
{
	library(ROracle)
	#p = paste("create table ",outTable, " as select ", featureNames, " from ",  inpTable, sep="")
	p = paste("create table ",outTable, " as select ", featureNames, " from ",  inpTable, " where ", predicate, sep="")
	
	#print(p)
	t = system.time((dbGetQuery(mconn, p)))
	
}
bInitMatSize <- function(str) {
	library(ROracle)
	t <- system.time((dbGetQuery(mconn, paste("select oreT.initMatSize(\'", str, "\') from dual", sep=""))))
	return(t)
}

bMaterialize = function(inpTable, featureNames, outTable)
{
	library(ROracle)

	qString <- paste("select count(*) from USER_TABLES where table_name = \'", toupper(outTable), "\'", sep = "")
	res <- dbGetQuery(mconn, qString)
	present <- res[[1]]
	if(present) {
		t <- dbGetQuery(mconn, paste("drop table " , outTable, sep=""))
	} 
	p = paste("create table ",outTable, " as select ", featureNames, " from ",  inpTable, sep="")
	#print(p)
	t = system.time((dbGetQuery(mconn, paste("create table ",outTable, " as select ", featureNames, " from ",  inpTable, sep=""))))
	
}
bInitMatSize <- function(str) {
	library(ROracle)
	t <- system.time((dbGetQuery(mconn, paste("select oreT.initMatSize(\'", str, "\') from dual", sep=""))))
	return(t)
}
bInitInStages <- function(str)
{
	library(ROracle)
	tot.time <- 0
	##print(str)
	chopLength = 3000
	numChops = ceiling(nchar(str)/chopLength)
	#print(paste("NumChops : ", numChops))
	#print(paste("lengthofstr : ", nchar(str)))
	
	resString = list()
	for(i in c(1:numChops))
	{
		s = substr(str, (i-1)*chopLength, min(nchar(str), (i)*chopLength))
		##print(s)
		resString[[i]] = substr(str, ((i-1)*chopLength) + 1, min(nchar(str), i*chopLength))
	}
	##print(paste("resString : ", resString))
	
	for(i in 1:numChops)
	{
		initStr = ""
		if(i == 1)
		{
			if(i == numChops)
			{
				initStr = paste("STE%", resString[[i]], sep="")	
			}
			else
			{
				initStr = paste("STM%", resString[[i]], sep="")
			}
		}
		else
		{
			if(i != numChops)
			{
				initStr = paste("MRE%", resString[[i]], sep="")
			}
			else
			{
				initStr = paste("END%", resString[[i]], sep="")
			}
		}
#		print(initStr)
		res <- system.time(dbGetQuery(mconn, paste("select
oreT.initializeInStages(\'",initStr,"\') from dual", sep="")), gcFirst = TRUE)
		tot.time <- tot.time + res
		
	}
	return(tot.time)
	
}
bResultInStages = function(strObj)
{
	library(ROracle)
	res = ""
	tmp = ""
	while(TRUE)
	{
		tmp = dbGetQuery(mconn, paste("select oreT.getResultInStages(\'",strObj,"\') from dual", sep=""))
		strTmp = as.character(tmp)
		##print(strTmp)
		splitted = strsplit(strTmp, split="%")
		tok = splitted[[1]][1]
		if(tok == "MRE")
		{
			res = paste(res, splitted[[1]][2], sep="") # splitted[[1]][2] contains the actual string.
		
		}
		else if(tok == "END")
		{
			res = paste(res, splitted[[1]][2], sep="")
			break
		}
		else
		{
			retur( NULL)
		}
		
	}
	return(res)
}
#--------------------------------------------------------------------------------

bUpdateSessionInfo <- function(session.id, user.id) {

	library(ROracle)
	query <- paste("INSERT INTO MYSESSIONINFO(SESSIONID, USERID, SESSION_DATE) VALUES(", session.id, ", " , user.id, ", sysdate)", sep = "")
	res <- dbGetQuery(mconn, query)
	res <- dbGetQuery(mconn, "commit")
	return(res)
	
}

bUpdateProgramDataset <- function(program.id, dataset.id) {
	
	library(ROracle)
	query <- paste0("INSERT INTO DATASET_PROGRAM VALUES(", dataset.id, ", ", program.id, ")")
	res <- dbGetQuery(mconn, query)
	res <- dbGetQuery(mconn, "commit")
	print(query)
	return(res)
}

bUpdateProgramInfo <- function(session.id, program.id, program.name, program.text) {
	
	library(ROracle)
    	query <- paste("INSERT INTO PROGRAM_INFO VALUES(", program.id, ", ", session.id, ", \'",
program.name, "\', \'", program.text, "\', sysdate)", sep = "")   
	#print(query)
	res <- dbGetQuery(mconn, query)
	res <- dbGetQuery(mconn, "commit")
	return(res)
}


bGetPgmTextFromName <- function(program.name) {
	library(ROracle)
	query <- paste("SELECT PROGRAM_TEXT FROM PROGRAM_INFO WHERE PROGRAM_NAME = \'",
program.name, "\'", sep="")
	res <- dbGetQuery(mconn, query)
	return(res$PROGRAM_TEXT)
}

bUpdateMdbTableNames <- function(program.id, session.id, program.name) {
	library(ROracle)
	
	res <- GetMDBTableNames(program.id, session.id, program.name)
	

	query <- paste0("INSERT INTO MDBINFO(PROGRAM_ID, SESSION_ID, PROGRAM_NAME, BEST_K_CONFIG, CG_CONFIG, IGD_CONFIG, DATASET, DERIVED_DATASETS, FEATURES, DERIVED_FEATURES, FEATUREMAP_DB, FEATURESET_DB, PROGRAM) VALUES(", program.id, ", ", session.id, ", ", SingleQuoteWrap(program.name), ", ", res$best.k.config, ", ", res$cg.config, ", ", res$igd.config, ", ", res$dataset, ", ", res$derived.datasets, ", ", res$features, ", ", res$derived.features, ", ", res$featuremap.db, ", ", res$featureset_db, ", ", res$program, ")")
	 print(query)
	res <- dbGetQuery(mconn, query)
	res <- dbGetQuery(mconn, "commit")

}

bDeleteTableNames <- function(program.id) {
	library(ROracle)
	query <- paste0("DELETE FROM MDBINFO WHERE PROGRAM_ID=", program.id)
	res <- dbGetQuery(mconn, query)
	res <- dbGetQuery(mconn, "commit")
	return(res)
}

bInsert <- function(table.name, column.names, vals) {
	library(ROracle)
	
	query <- ""
	
	# fill the table name
	query <- paste0("INSERT INTO ", table.name, "(")
	
	# fill in the column. names
	column.str <- paste(column.names, collapse=",")
	
	query <- paste0(query, column.str, ") VALUES(")

	# fill in the values
	val.str <- paste(vals, collapse=",")

	query <- paste0(query, val.str, ")")
	#print(query)
	res <- dbGetQuery(mconn, query)
	res <- dbGetQuery(mconn, "commit")
	return(res)
		
} 

bCount <- function(table.name) {
	library(ROracle)
	query <- paste0("SELECT COUNT(*) AS CNT FROM ", table.name)
	res <- dbGetQuery(mconn, query)
	return(res$CNT)
}

bSelect <- function(ret.column.name, table.name, check.column.name, check.value, ...) {
	library(ROracle)
	query <- paste0("SELECT ", ret.column.name, " FROM ", table.name, " WHERE ", check.column.name, " = ", check.value)
	
	nargs <- list(...)
	if(length(nargs) > 0 ) {
		for(i in seq(1, length(nargs), 2)) {
			query <- paste0(query, " AND ")
			query <- paste0(query, nargs[[i]], " = ", nargs[[i + 1]])
		}
	}
	
	#print(query)
	res <- dbGetQuery(mconn, query)
	return(res)
}

bDeleteTuples <- function(table.name) {
	library(ROracle)
	query <- paste0("DELETE FROM ", table.name)
	res <- dbGetQuery(mconn, query)
	res <- dbGetQuery(mconn, "commit")
	return(res)
}

bMaxId <- function(table.name) {
	library(ROracle)
	query <- paste0("SELECT MAX(ID) AS MID FROM ", table.name)
	res <- dbGetQuery(mconn, query)
	return(res$MID)
}
bMaxValues <- function(table.name, attr.vec) {
	len <- length(attr.vec)
	query <- "SELECT " 
	for(index in 1:len) {
		query <- paste0(query, "MAX(", attr.vec[[index]], ") AS ", paste0("MX", index), " ")
		if(index != len)
			query <- paste0(query, ", ")
	}
	query <- paste0(query, " FROM ", table.name)
	res <- dbGetQuery(mconn, query)
	return(res)
}
bMinValues <- function(table.name, attr.vec) {
	len <- length(attr.vec)
	query <- "SELECT " 
	for(index in 1:len) {
		query <- paste0(query, "MIN(", attr.vec[[index]], ") AS ", paste0("MX", index), " ")
		if(index != len)
			query <- paste0(query, ", ")
	}
	query <- paste0(query, " FROM ", table.name)
	res <- dbGetQuery(mconn, query)
	return(res)
}
bGetCompatiblePrograms <- function(dataset.id) {
	query <- paste0("SELECT PROGRAM_NAME, A.PROGRAM_ID FROM PROGRAM_INFO A, DATASET_PROGRAM B WHERE A.PROGRAM_ID = B.PROGRAM_ID AND DATASET_ID = ", dataset.id)
	print(query)
	res <- dbGetQuery(mconn, query)
	
	return(res)
}

bGetDatasets <- function() {
	library(ROracle)
	query <- paste0("SELECT DATASETID, TABLE_NAME, INIT_NUMFEATURES, TARGET_FEATURE_INDEX FROM DATASET ORDER BY DATASETID")
	res <- dbGetQuery(mconn, query)
	return(res)
}
bExecuteQuery <- function(query) {
	library(ROracle)
	res <- dbGetQuery(mconn, query)
	res <- dbGetQuery(mconn, "commit")
}

bGetTablesInDateRange <- function(dataset.name, st_date, en_date) {
	library(ROracle)
	query <- paste0("SELECT TABLE_NAME, INIT_NUMFEATURES, TARGET_FEATURE_INDEX, to_char(DATE_CREATED, 'mm/dd/yyyy'), NUM_TUPLES, NUM_TIMES_ACCESSED FROM DATASET where date_created >= to_date(\'", st_date,"\','mm/dd/yyyy') and date_created <= to_date(\'", en_date, "\', 'mm/dd/yyyy') and TABLE_NAME = \'", dataset.name, "\'")
	print(query)
	res <- dbGetQuery(mconn, query)
	return(res)
}

bGetNumTimesAccessed <- function(table.name) {
	library(ROracle)
	query <- paste0("select num_times_accessed from dataset where table_name = \'", table.name, "\'")
	res <- dbGetQuery(mconn, query)
}

bUpdateNumTimesAccessed <- function(table.name, num.times.accessed) {
	library(ROracle)
	query <- paste0("update dataset set num_times_accessed = ", num.times.accessed, " where table_name = \'", table.name, "\'")
	bExecuteQuery(query) 
}

