SetUpForStepAddIgdTrain <- function(step.num) {
	
	inp.feat.id <- GetInputSetId.1(step.num)
	inp.set.val <- unlist(GetFeatureSet(inp.feat.id))
	
	inp.mask.id <- GetInputSetId.2(step.num)
	mask.set.val <- unlist(GetFeatureSet(inp.mask.id))
	
	dataset.id <- GetDatasetIdFromStepNum(step.num)
	glb.feat.indices <- GetFeatureIndices(dataset.id)
	
	temp.set <- setdiff(setdiff(glb.feat.indices, mask.set.val), inp.set.val)
	
	feat.set.list <- VectorOneJoin(inp.set.val, temp.set)
	
	igd.config <- GetIgdConfig(step.num)
	
	decay <- igd.config$decay
	step.size <- igd.config$step.size
	num.folds <- igd.config$num.folds
	
	inp.list.id <- GetInputList.1(step.num)
	
	if(inp.list.id == MNULL) {
		init.wt <- igd.config$init.wt
		coeff.list <- list()
	
		for(i in 1:length(feat.set.list)) {
			coeff.list[[i]] <- list()
			for(j in 1:num.folds) {
				coeff.list[[i]][[j]] <- list()
				coeff.list[[i]][[j]] <- rep(init.wt,
length(unlist(feat.set.list[[i]])))
			}
		}
	} else {
			values <- GetFeatureMapValues(inp.list.id)
			values <- values[[1]]
			coeff.list <- values
	}
	
	param.vec <- c(step.size, decay, num.folds)
	res <- InitIgdCoeffLearner(param.vec, feat.set.list, coeff.list)
	return(res)
	
	
}
SetUpForStepAddCgTrain <- function(step.num) {

	inp.feat.id <- GetInputSetId.1(step.num)
	inp.set.val <- unlist(GetFeatureSet(inp.feat.id))
	
	inp.mask.id <- GetInputSetId.2(step.num)
	mask.set.val <- unlist(GetFeatureSet(inp.mask.id))
	
	dataset.id <- GetDatasetIdFromStepNum(step.num)
	glb.feat.indices <- GetFeatureIndices(dataset.id)
	
	temp.set <- setdiff(setdiff(glb.feat.indices, mask.set.val), inp.set.val)
	
	feat.set.list <- VectorOneJoin(inp.set.val, temp.set)
	
	cg.config <- GetCgConfig(step.num)
	num.folds <- cg.config$num.folds
	inp.list.id <- GetInputList.1(step.num)

	# default values.
	def.u <- 0
	def.g <- 0
	
	if(inp.list.id == MNULL) {
		init.wt <- cg.config$init.wt
		coeff.list <- list()
		g.list <- list()
		u.list <- list()
		for(i in 1:length(feat.set.list)) {
			coeff.list[[i]] <- list()
			u.list[[i]] <- list()
			g.list[[i]] <- list()
			len.feat.set <- length(unlist(feat.set.list[[i]]))
			u.val <- rep(def.u, len.feat.set)
			g.val <- rep(def.g, len.feat.set)
			wt.val <- rep(init.wt, len.feat.set)
			for(j in 1:num.folds) {
				coeff.list[[i]][[j]] <- list()
				u.list[[i]][[j]] <- list()
				g.list[[i]][[j]] <- list()

				coeff.list[[i]][[j]] <- wt.val
				g.list[[i]][[j]] <- g.val
				u.list[[i]][[j]] <- u.val
				
			}
		}
		
		
	} else {
		values <- GetFeatureMapValues(inp.list.id)
		values <- values[[1]]
		
		coeff.list <- values$coeff.list
		u.list <- values$u.list
		g.list <- values$g.list
	}
	param.vec <- c(num.folds)
	res <- InitCgCoeffLearner(param.vec, feat.set.list, coeff.list, u.list, g.list)
	return(res)
}

SetUpForStepAddCvTest <- function(step.num) {
	inp.feat.id <- GetInputSetId.1(step.num)
	inp.set.val <- unlist(GetFeatureSet(inp.feat.id))
	
	inp.mask.id <- GetInputSetId.2(step.num)
	mask.set.val <- unlist(GetFeatureSet(inp.mask.id))
	
	dataset.id <- GetDatasetIdFromStepNum(step.num)
	glb.feat.indices <- GetFeatureIndices(dataset.id)
	
	temp.set <- setdiff(setdiff(glb.feat.indices, mask.set.val), inp.set.val)
	
	feat.set.list <- VectorOneJoin(inp.set.val, temp.set)

	config <- GetIgdConfig(step.num, exit.flag = FALSE)
	is.cg <- FALSE
	if((is.list(config) == FALSE) && (config == MNULL)) {
		config <- GetCgConfig(step.num)
		is.cg <- TRUE
	}

	num.folds <- config$num.folds
	inp.list.id <- GetInputList.1(step.num)
	
	if(inp.list.id == MNULL) {
		init.wt <- config$init.wt
		coeff.list <- list()
	
		for(i in 1:length(feat.set.list)) {
			coeff.list[[i]] <- list()
			for(j in 1:num.folds) {
				coeff.list[[i]][[j]] <- list()
				coeff.list[[i]][[j]] <- rep(init.wt,
length(unlist(feat.set.list[[i]])))
			}
		}
	} else {
			values <- GetFeatureMapValues(inp.list.id)
			values <- values[[1]]
			coeff.list <- values
	}
		
	res <- InitEvalCvTest(num.folds, feat.set.list, coeff.list, is.cg)	
	return(res)
	

}
SetUpForStepAddAiCTest <- function(step.num) {
	inp.feat.id <- GetInputSetId.1(step.num)
	inp.set.val <- unlist(GetFeatureSet(inp.feat.id))
	
	inp.mask.id <- GetInputSetId.2(step.num)
	mask.set.val <- unlist(GetFeatureSet(inp.mask.id))
	
	dataset.id <- GetDatasetIdFromStepNum(step.num)
	glb.feat.indices <- GetFeatureIndices(dataset.id)
	
	temp.set <- setdiff(setdiff(glb.feat.indices, mask.set.val), inp.set.val)
	
	feat.set.list <- VectorOneJoin(inp.set.val, temp.set)

	config <- GetIgdConfig(step.num, exit.flag = FALSE)
	is.cg <- FALSE
	if((is.list(config) == FALSE) && (config == MNULL)) {
		config <- GetCgConfig(step.num)
		is.cg <- TRUE
	}

	num.folds <- config$num.folds
	inp.list.id <- GetInputList.1(step.num)
	
	if(inp.list.id == MNULL) {
		init.wt <- config$init.wt
		coeff.list <- list()
	
		for(i in 1:length(feat.set.list)) {
			coeff.list[[i]] <- list()
			for(j in 1:num.folds) {
				coeff.list[[i]][[j]] <- list()
				coeff.list[[i]][[j]] <- rep(init.wt,
length(unlist(feat.set.list[[i]])))
			}
		}
	} else {
			values <- GetFeatureMapValues(inp.list.id)
			values <- values[[1]]
			coeff.list <- values
	}
		
	res <- InitEvalAiCTest(num.folds, feat.set.list, coeff.list, is.cg)	
	return(res)
	

}

ResStepAddIgdTrain <- function(step.num, res.string) {
	ResEvaluateIgdTrain(step.num, res.string)
	
}
ResStepAddCgTrain <- function(step.num, res.string) {
	exe.list <- SetUpForStepAddCgTrain(step.num)
	num.folds <- exe.list$param.vec[1]

	list.id <- GetOutputListId(step.num)
	res.list <- ParseCgCoeffLearner(res.string, num.folds, exe.list$feat.set.list,
exe.list$coeff.list, exe.list$u.list, exe.list$g.list)

	out.list <- list()
	out.list$coeff.list <- res.list$coeff.list
	out.list$u.list <- res.list$u.list
	out.list$g.list <- res.list$g.list

	SetFeatMapFeatSetList(list.id, res.list$feat.set.list)
	SetFeatureMapValues(list.id, out.list)
}

ResStepAddCvTest <- function(step.num, res.string) {
	exe.list <- SetUpForStepAddCvTest(step.num)
	num.folds <- exe.list$param.vec[1]
	
	list.id <- GetOutputListId(step.num)
	res.list <- ParseIgdCoeffLearner(res.string, num.folds)
	SetFeatMapFeatSetList(list.id, res.list$feat.set.list)
	coeff.list <- lapply(lapply(res.list$coeff.list, unlist), mean)
	SetFeatureMapValues(list.id, coeff.list)
}

ResStepAddAiCTest <- function(step.num, res.string) {
	exe.list <- SetUpForStepAddAiCTest(step.num)
	len.param.vec <- length(exe.list$param.vec)
	
	if(len.param.vec == 3)	
		num.folds <- exe.list$param.vec[3]
	else
		num.folds <- exe.list$param.vec[1]

	list.id <- GetOutputListId(step.num)
	res.list <- ParseIgdCoeffLearner(res.string, num.folds)
	SetFeatMapFeatSetList(list.id, res.list$feat.set.list)
	coeff.list <- lapply(lapply(res.list$coeff.list, unlist), mean)
	SetFeatureMapValues(list.id, coeff.list)
}
ExeStepAddBestK <- function(step.num) {
	k.val <- 1
	
	prev.op.id <- GetOperatorId(step.num - 1)

	if(prev.op.id == STEPADD_EVAL_CV_TEST) # pick the set with lowest CV
		rev.flag = FALSE
	else
		rev.flag = TRUE

	list.id <- GetInputList.1(step.num)
	feat.list <- GetFeatureMapFeatureSet(list.id)[[1]]
	values <- GetFeatureMapValues(list.id)[[1]]
	
	rnk <- rank(unlist(values), ties.method = "first")
	
	if(rev.flag == TRUE)
		rnk <- rev(rnk)
	
	ret.set <- unlist(feat.list[[rnk[1]]])
	out.feat.id <- GetOutputSetId(step.num)
	SetFeatureSet(out.feat.id, ret.set)
}	

#-----------------------------------------------------

SetUpForStepDelIgdTrain <- function(step.num) {
	
	inp.feat.id <- GetInputSetId.1(step.num)
	inp.set.val <- unlist(GetFeatureSet(inp.feat.id))

	feat.set.list <- combn(inp.set.val, length(inp.set.val) - 1, simplify = FALSE)
	
	igd.config <- GetIgdConfig(step.num)

	decay <- igd.config$decay
	step.size <- igd.config$step.size
	num.folds <- igd.config$num.folds

	inp.list.id <- GetInputList.1(step.num)
				
	if(inp.list.id == MNULL) {
		init.wt <- igd.config$init.wt
		coeff.list <- list()
	
		for(i in 1:length(feat.set.list)) {
			coeff.list[[i]] <- list()
			for(j in 1:num.folds) {
				coeff.list[[i]][[j]] <- list()
				coeff.list[[i]][[j]] <- rep(init.wt,
length(unlist(feat.set.list[[i]])))
			}
		}
	} else {
			values <- GetFeatureMapValues(inp.list.id)
			values <- values[[1]]
			coeff.list <- values
	}
	
	param.vec <- c(step.size, decay, num.folds)
	res <- InitIgdCoeffLearner(param.vec, feat.set.list, coeff.list)
	return(res)
	
}

SetUpForStepDelCgTrain <- function(step.num) {
	inp.feat.id <- GetInputSetId.1(step.num)
	inp.set.val <- unlist(GetFeatureSet(inp.feat.id))

	feat.set.list <- combn(inp.set.val, length(inp.set.val) - 1, simplify = FALSE)
	cg.config <- GetCgConfig(step.num)
	num.folds <- cg.config$num.folds
	inp.list.id <- GetInputList.1(step.num)

	# default values.
	def.u <- 0
	def.g <- 0
	
	if(inp.list.id == MNULL) {
		init.wt <- cg.config$init.wt
		coeff.list <- list()
		g.list <- list()
		u.list <- list()
		for(i in 1:length(feat.set.list)) {
			coeff.list[[i]] <- list()
			u.list[[i]] <- list()
			g.list[[i]] <- list()
			len.feat.set <- length(unlist(feat.set.list[[i]]))
			u.val <- rep(def.u, len.feat.set)
			g.val <- rep(def.g, len.feat.set)
			wt.val <- rep(init.wt, len.feat.set)
			for(j in 1:num.folds) {
				coeff.list[[i]][[j]] <- list()
				u.list[[i]][[j]] <- list()
				g.list[[i]][[j]] <- list()

				coeff.list[[i]][[j]] <- wt.val
				g.list[[i]][[j]] <- g.val
				u.list[[i]][[j]] <- u.val
				
			}
		}
		
		
	} else {
		values <- GetFeatureMapValues(inp.list.id)
		values <- values[[1]]
		
		coeff.list <- values$coeff.list
		u.list <- values$u.list
		g.list <- values$g.list
	}
	param.vec <- c(num.folds)
	res <- InitCgCoeffLearner(param.vec, feat.set.list, coeff.list, u.list, g.list)
	return(res)
	
}

SetUpForStepDelCvTest <- function(step.num) {
	inp.feat.id <- GetInputSetId.1(step.num)
	inp.set.val <- unlist(GetFeatureSet(inp.feat.id))

	feat.set.list <- combn(inp.set.val, length(inp.set.val) - 1, simplify = FALSE)
	
	config <- GetIgdConfig(step.num, exit.flag = FALSE)
	is.cg <- FALSE
	if((is.list(config) == FALSE) && (config == MNULL)) {
		config <- GetCgConfig(step.num)
		is.cg <- TRUE
	}

	num.folds <- config$num.folds
	inp.list.id <- GetInputList.1(step.num)
	
	if(inp.list.id == MNULL) {
		init.wt <- config$init.wt
		coeff.list <- list()
	
		for(i in 1:length(feat.set.list)) {
			coeff.list[[i]] <- list()
			for(j in 1:num.folds) {
				coeff.list[[i]][[j]] <- list()
				coeff.list[[i]][[j]] <- rep(init.wt,
length(unlist(feat.set.list[[i]])))
			}
		}
	} else {
			values <- GetFeatureMapValues(inp.list.id)
			values <- values[[1]]
			coeff.list <- values
	}
		
	res <- InitEvalCvTest(num.folds, feat.set.list, coeff.list, is.cg)	
	return(res)

}

SetUpForStepDelAiCTest <- function(step.num) {
	inp.feat.id <- GetInputSetId.1(step.num)
	inp.set.val <- unlist(GetFeatureSet(inp.feat.id))

	feat.set.list <- combn(inp.set.val, length(inp.set.val) - 1, simplify = FALSE)

	config <- GetIgdConfig(step.num, exit.flag = FALSE)
	is.cg <- FALSE
	if((is.list(config) == FALSE) && (config == MNULL)) {
		config <- GetCgConfig(step.num)
		is.cg <- TRUE
	}

	num.folds <- config$num.folds
	inp.list.id <- GetInputList.1(step.num)
	
	if(inp.list.id == MNULL) {
		init.wt <- config$init.wt
		coeff.list <- list()
	
		for(i in 1:length(feat.set.list)) {
			coeff.list[[i]] <- list()
			for(j in 1:num.folds) {
				coeff.list[[i]][[j]] <- list()
				coeff.list[[i]][[j]] <- rep(init.wt,
length(unlist(feat.set.list[[i]])))
			}
		}
	} else {
			values <- GetFeatureMapValues(inp.list.id)
			values <- values[[1]]
			coeff.list <- values
	}
		
	res <- InitEvalAiCTest(num.folds, feat.set.list, coeff.list, is.cg)	
	return(res)
	
}

ResStepDelIgdTrain <- function(step.num, res.string) {
	ResEvaluateIgdTrain(step.num, res.string)
}

ResStepDelCgTrain <- function(step.num, res.string) {
	
	exe.list <- SetUpForStepDelCgTrain(step.num)
	num.folds <- exe.list$param.vec[1]

	list.id <- GetOutputListId(step.num)
	res.list <- ParseCgCoeffLearner(res.string, num.folds, exe.list$feat.set.list,
exe.list$coeff.list, exe.list$u.list, exe.list$g.list)

	out.list <- list()
	out.list$coeff.list <- res.list$coeff.list
	out.list$u.list <- res.list$u.list
	out.list$g.list <- res.list$g.list

	SetFeatMapFeatSetList(list.id, res.list$feat.set.list)
	SetFeatureMapValues(list.id, out.list)
	
}

ResStepDelCvTest <- function(step.num, res.string) {
	exe.list <- SetUpForStepDelCvTest(step.num)
	num.folds <- exe.list$param.vec[1]
	list.id <- GetOutputListId(step.num)
	res.list <- ParseIgdCoeffLearner(res.string, num.folds)
	SetFeatMapFeatSetList(list.id, res.list$feat.set.list)
	coeff.list <- lapply(lapply(res.list$coeff.list, unlist), mean)
	SetFeatureMapValues(list.id, coeff.list)
}

ResStepDelAiCTest <- function(step.num, res.string) {
	
	exe.list <- SetUpForStepDelAiCTest(step.num)
	len.param.vec <- length(exe.list$param.vec)
	
	if(len.param.vec == 3)	
		num.folds <- exe.list$param.vec[3]
	else
		num.folds <- exe.list$param.vec[1]

	list.id <- GetOutputListId(step.num)
	res.list <- ParseIgdCoeffLearner(res.string, num.folds)
	SetFeatMapFeatSetList(list.id, res.list$feat.set.list)
	coeff.list <- lapply(lapply(res.list$coeff.list, unlist), mean)
	SetFeatureMapValues(list.id, coeff.list)
	
}

ExeStepDelBestK <- function(step.num) {
	ExeStepAddBestK(step.num)
}

