# This file contains the general purpose helper functions.

# Define printf function
printf <- function(...) print(sprintf(...))

# Logger function
mlog <- function(..., file = 'log.txt') {
	# Get the string object 
	str.obj <- printf(...)
	write(str.obj, file = file, append = TRUE)
}

VectorJoin <- function(m1, m2) {
	ret.list <- list()
	ind <- 1

	for(i in m1) {
		for(j in m2) {
			if(i != j) {
				ret.list[[ind]] <- sort(c(i, j))
				ind <- ind + 1
			}
		}
	}
	
	return(ret.list)
}

VectorOneJoin <- function(m1, m2) {

	ret.list <- list()
	ind <- 1
	
	for(j in m2) {
		ret.list[[ind]] <- unique(sort(c(m1, j)))
		ind <- ind + 1
	}
	return(ret.list)
	
}

NumVec2Str <- function(feat.set, sep=",") {
	str <- paste(as.character((unlist(feat.set))), collapse = sep)
	return(str)
}

Str2NumVec <- function(str, sep=",") {
	vec <- unlist(lapply(strsplit(str, split=sep), as.numeric))
	return(vec)
}

SingleQuoteWrap <- function(str) {
	return(paste0("'", str, "'"))
}
GetFeatureMapDBFilenames <- function(prefix.str, single.quote = TRUE) {
	num.rows <- GetNumFeatureMapEntries()
	ret.file.names <- list()
	for(n in 1:num.rows) {
		ret.file.names[[n]] <- paste(prefix.str, "featuremapdb", n, sep = "_")
		if(single.quote)
			ret.file.names[[n]] <- SingleQuoteWrap(ret.file.names[[n]])
	}
	return(ret.file.names)
}

GetMDBTableNames <- function(program.id, session.id, program.name, dataset.id, single.quote = TRUE) {
	prefix.str <- paste(program.name, dataset.id, sep="_")

	res <- list()
	res$best.k.config <- paste(prefix.str, "bestkconfig", sep="_")
	res$cg.config <- paste(prefix.str, "cgconfig", sep="_")
	res$dataset <- paste(prefix.str, "dataset", sep="_")
	res$derived.datasets <- paste(prefix.str, "derdatasets", sep="_")
	res$derived.features <- paste(prefix.str, "derfeatures", sep="_")
	res$features <- paste(prefix.str, "features", sep="_")
	res$featureset_db <- paste(prefix.str, "featuresetdb", sep="_")
	res$igd.config <- paste(prefix.str, "igdconfig", sep="_")
	res$program <- paste(prefix.str, "program", sep="_")
	
	if(single.quote == TRUE) {
		res$best.k.config <- SingleQuoteWrap(res$best.k.config)
		res$cg.config <- SingleQuoteWrap(res$cg.config)
		res$dataset <- SingleQuoteWrap(res$dataset)
		res$derived.datasets <- SingleQuoteWrap(res$derived.datasets)
		res$derived.features <- SingleQuoteWrap(res$derived.features)
		res$featuremap.db <- SingleQuoteWrap(res$featuremap.db)
		res$features <- SingleQuoteWrap(res$features)
		res$featureset_db <- SingleQuoteWrap(res$featureset_db)
		res$igd.config <- SingleQuoteWrap(res$igd.config)
		res$program <- SingleQuoteWrap(res$program)
		
	}
	res$featuremap_db <- GetFeatureMapDBFilenames(prefix.str, single.quote)
	
	
	return(res)

	
	
}

is.valid.val <- function(num) {
	
	if(is.na(num) || is.nan(num)) 
		return(FALSE)
	return(TRUE)
}

GetValidValue <- function(num) {
	if(is.valid.val(num) == FALSE)
		return(0.453)
	return(num)
}

FillAttrnames <- function(table.name, skip.first = TRUE) {
	
	attr.names <- GetAttrnames(table.name)
	len.attr.names <- length(attr.names)
	if(skip.first)
		column.names <- as.list(attr.names[2:len.attr.names]) # leave the id
	else
		column.names <- as.list(attr.names)

	return(column.names)
}

DeleteAllOpTables <- function() {
	
	bDeleteTuples("CORRELATIONX")
	bDeleteTuples("CORRELATIONY")
	bDeleteTuples("EVAL_CV_AIC")
	bDeleteTuples("CG_COEFF")
	bDeleteTuples("CG_CONFIG")
	bDeleteTuples("IGD_COEFF")
	bDeleteTuples("IGD_CONFIG")
}

VectorMatch <- function(v1, v2) {
	v1 <- unlist(v1)
	v2 <- unlist(v2)

	if(length(v1) != length(v2))
		return(FALSE)
	return(all(v1 == v2))
}
ApproxVectorMatch <- function(v1, v2) {
	th <- 1e-5
	v1 <- unlist(v1)
	v2 <- unlist(v2)
	if(length(v1) != length(v2))
		return(FALSE)

	idx <- which(abs(v1 - v2) > th)
	if(length(idx) > 0)
		return(FALSE)
	else
		return(TRUE)
	
}


