PlotResults <- function(stats.list) {
	g1 <- GetNPlotDF(stats.list)
	p1 <-ggplot(g1, aes(x = factor(Batch), y = Time.Taken, fill = factor(OpType))) + geom_bar(position = "fill", stat = "identity") + xlab("Batch Number") + ylab("Fractional Time Taken") + ggtitle("Runtime")
	g2 <- GetNPlotOpTypeFreqsDF(stats.list)
	p2 <- ggplot(g2, aes(x = factor(Batch), y = NumOperations, fill = factor(OpClass))) + geom_bar(position = "fill", stat = "identity") + xlab("Batch Number") + ylab("Fraction of Operations") + ggtitle("Operations in each Batch")
	multiplot(p1, p2)
}
multiplot <- function(..., plotlist=NULL, file, cols=1, layout=NULL) {
  require(grid)

  # Make a list from the ... arguments and plotlist
  plots <- c(list(...), plotlist)

  numPlots = length(plots)

  # If layout is NULL, then use 'cols' to determine layout
  if (is.null(layout)) {
    # Make the panel
    # ncol: Number of columns of plots
    # nrow: Number of rows needed, calculated from # of cols
    layout <- matrix(seq(1, cols * ceiling(numPlots/cols)),
                    ncol = cols, nrow = ceiling(numPlots/cols))
  }

 if (numPlots==1) {
    print(plots[[1]])

  } else {
    # Set up the page
    grid.newpage()
    pushViewport(viewport(layout = grid.layout(nrow(layout), ncol(layout))))

    # Make each plot, in the correct location
    for (i in 1:numPlots) {
      # Get the i,j matrix positions of the regions that contain this subplot
      matchidx <- as.data.frame(which(layout == i, arr.ind = TRUE))

      print(plots[[i]], vp = viewport(layout.pos.row = matchidx$row,
                                      layout.pos.col = matchidx$col))
    }
  }
}
PlotDag <- function(dat.frame, feat.name = NULL) {
	g <- graph.data.frame(dat.frame)
	d <- which(dat.frame[, 'out.str'] == 1)
	E(g)$color <- "grey"
	E(g)$width <- 1
	
	E(g)[d]$color <- "red"
	E(g)[d]$width <- 2

	V(g)$label.color <- "blue"
	V(g)$color <- "SkyBlue2"
	
	d <- setdiff(d, 10)
	V(g)[d]$label.color <- "black"
	V(g)[d]$color <- "red"
	#plot(g, layout=layout.fruchterman.reingold, vertex.label.dist = 0, vertex.size = 70,vertex.shape = "rectangle", asp=0, vertex.label.cex=0.7, vertex.label.family = 'NimbusSan')
	plot(g, layout=layout.circle, vertex.label.dist = 0, vertex.size = 70,vertex.shape = "rectangle", asp=0, vertex.label.cex=0.55, vertex.label.family = 'NimbusSan')
	title(main = paste0("DAG : ", feat.name))
	axis(1, labels = FALSE, tick = TRUE)
	axis(2, labels = FALSE, tick = TRUE)
 	
}

is.displayable <- function(step.num) {
	op.id <- GetOperatorId(step.num)
	return(!op.id %in% c("COEF_CG_TRAIN", "EVAL_CG_TRAIN", "STEPADD_EVAL_CG_TRAIN", "STEPDEL_EVAL_CG_TRAIN","EVAL_ID_TRAIN", "STEPADD_EVAL_IGD_TRAIN", "STEPDEL_EVAL_IGD_TRAIN"))
}

GetOutputString <- function(step.num) {
	# Check whether there is a set output
	out.id <- GetOutputSetId(step.num)
	if(out.id != MNULL) {
		out.set <- unlist(GetFeatureSet(out.id))
		out.str <- paste0("", paste(out.set, collapse = ","), "")
	} else {
		out.id <- GetOutputListId(step.num)
		out.list <- GetFeatureMapValues(out.id)[[1]]
		#print(step.num)
		if(is.displayable(step.num)) {
			out.list <- unlist(out.list)
			out.str <- paste0("", paste(out.list, collapse=","), "")
			out.str <- ''
		}  else {
			out.str <- ''
		}

	}
		
	return(out.str)
	
	
}

RenameOpId <- function(op.id) {
	if(op.id == "CORRELATION_X")
		return("CORRX")
	else if(op.id == "CORRELATION_Y")
		return("CORRY")
	else if(op.id == "COEF_IGD_TRAIN")
		return("COEF_IGD")
	else if(op.id == "COEF_CG_TRAIN")
		return("COEF_CG")
	else if (op.id == "DEL_FEATURE")
		return("DEL_FS")
	else if(op.id == "EVAL_IGD_TRAIN")
		return("IGD_TR")
	else if(op.id == "EVAL_CG_TRAIN")
		return("CG_TR")
	else if(op.id == "EVAL_CV_TEST")
		return("CV_TS")
	else if(op.id == "EVAL_AIC_TEST")
		return("AIC_TS")
	else if(op.id == "STEPADD_EVAL_CG_TRAIN")
		return("ADD_CG_TR")
	else if(op.id == "STEPADD_EVAL_IGD_TRAIN")
		return("ADD_IGD_TR")
	else if(op.id == "STEPADD_EVAL_AIC_TEST")
		return("ADD_AIC_TS")
	else if(op.id == "STEPADD_EVAL_CV_TEST")
		return("ADD_CV_TS")
	else if(op.id == "STEPADD_BEST_K")
		return("ADD_K")
	else if(op.id == "STEPDEL_EVAL_CG_TRAIN")
		return("DEL_CG_TR")
	else if(op.id == "STEPDEL_EVAL_IGD_TRAIN")
		return("DEL_IGD_TR")
	else if(op.id == "STEPDEL_EVAL_AIC_TEST")
		return("DEL_AIC_TS")
	else if(op.id == "STEPDEL_EVAL_CV_TEST")
		return("DEL_CV_TS")
	else if(op.id == "STEPDEL_BEST_K")
		return("DEL_K")
	else if(op.id == "FEATURE_SET")
		return("FEAT_SET")
	else if(op.id == "BEST_K")
		return("BEST_K")
	
	
}

GetOperatorDataframe <- function(node.list) {
	flag <- 0
	out.dataframe <- data.frame()
	count <- 0 
	for(node in node.list) {
		count <- count + 1
		op.name <- GetOperatorId(node$step.num)
		#op.name <- RenameOpId(op.name)
		op.name <- paste0(op.name, "[", count, "]")
		#out.str <- GetOutputString(node$step.num)
		
		if(is.null(node$parents)) {
		} else {
			for(p in node$parents) {
					
				out.str <- GetOutputString(p)
				
				parent.op.name <- GetOperatorId(p)
				#parent.op.name <- RenameOpId(parent.op.name)
				parent.op.name <- paste0(parent.op.name, "[", p, "]")
				
				if(count == length(node.list)) {
					if(nchar(out.str) == 0) {
						out.str <- GetOutputString(node$step.num)
					}
				}
				if(flag == 0) {
					#print("Flag 0 Case")
					out.dataframe <- data.frame(parent.op.name, op.name, out.str)
					names(out.dataframe) <- c("parent.op", "op.id", "out.str")	
					
					flag <- 1
				} else {
					#print("Flag 1 case")
					tmp.df <-  data.frame(parent.op.name, op.name, out.str)
					names(tmp.df) <- c("parent.op","op.id",  "out.str")	
					out.dataframe <- rbind(out.dataframe, tmp.df)
				}
			}
		}	
		
		
	}
	return(out.dataframe)
	
}

GetDag <- function( dataset.id) {
	library(igraph)
	node.list <- GenNodeList(dataset.id)
	df1 <- GetOperatorDataframe(node.list)
	PlotDag(df1)

	
}

# -- plot the program dag

GetPlotParamsForDataAgnosticOps <- function(node.list) {
	node.params <- c()
	op.names <- c()
	for(node in node.list) {
		if(!is.null(node) && node$data.aware == 0) {
			op.name <- GetOperatorId(node$step.num)
			op.name <- RenameOpId(op.name)
			op.name <- paste0(op.name,"[",node$step.num, "]")	
			op.names <- c(op.names, op.name)
			node.params <- c(node.params, "ellipse")
		}
	}
	names(node.params) <- op.names
	return(node.params)

}
GetGraphNodeList <- function(node.list) {
	graph.node.list <- c()
	for(node in node.list) {
    if(!is.null(node)) {
	  	#op.name <- GetOperatorId(node$step.num)
		  #op.name <- paste0(op.name,"[",node$step.num, "]")	
		  op.name <- GetOperatorId(node$step.num)
		  op.name <- RenameOpId(op.name)
		  op.name <- paste0(op.name,"[",node$step.num, "]")	
		  #op.name <- paste0(node$step.num)
		  graph.node.list <- c(graph.node.list, op.name)
	  }
	}
	return(graph.node.list)
	
}

GetEdgesList <- function(node.list) {
	
	edge.list <- list()
	for(node in node.list) {
    if(!is.null(node)) {
		  #op.name <- GetOperatorId(node$step.num)
		  #op.name <- paste0(op.name, "[", node$step.num, "]")
		  #op.name <- paste0(node$step.num)
		  op.name <- GetOperatorId(node$step.num)
		  op.name <- RenameOpId(op.name)
		  op.name <- paste0(op.name,"[",node$step.num, "]")	
  		
	  	edge.list[[op.name]] <- list()
  		node.edges <- c()
		  if(length(node.list[[node$step.num]]$children) > 0) {
			  for(child in node.list[[node$step.num]]$children) {
					
				  #child.op.name <- GetOperatorId(child)
				  #child.op.name <- paste0(child.op.name, "[", child, "]")
				  #child.op.name <- paste0(child)
				  child.op.name <- GetOperatorId(child)
				  child.op.name <- RenameOpId(child.op.name)
				  child.op.name <- paste0(child.op.name,"[",child, "]")	
				  node.edges <- c(node.edges, child.op.name)
			  }
			
			  edge.list[[op.name]][["edges"]] <- (node.edges)
		
  	  } else {
	    } 

	  }		
	}
	
	return(edge.list)
}



GetProgramDag <- function(pgm.name, dataset.id) {
	library(Rgraphviz)
	node.list <- GenNodeList(dataset.id)
	graph.node.list <- GetGraphNodeList(node.list)
	edges.list <- GetEdgesList(node.list)
	g <- new('graphNEL', nodes=graph.node.list, edgeL = edges.list, edgemode='directed')
	graph.par(list(nodes=list(col='darkgreen', lwd="2", fontsize="16", width=1.5, lty="solid")))
	g <- layoutGraph(g)
	pp <- GetPlotParamsForDataAgnosticOps(node.list)
	nodeRenderInfo(g) <- list(shape = "rectangle")
	nodeRenderInfo(g) <- list(shape=pp)
	attrs <- list(node = list(fixedsize=FALSE))
	g <- layoutGraph(g, attrs = attrs)
	pp.colr <- pp
	pp.colr <- rep("lightgray", length(pp))
	names(pp.colr) <- names(pp)
	#nodeRenderInfo(g) <- list(fill="yellow")
	nodeRenderInfo(g) <- list(fill=pp.colr)
	graph.par(list(graph=list(main="Program DAG without Batch optimization", cex.main=0.8)))
	return(g)
}

GetColorCodingForBatches <- function(node.list) {
	batch.stepnums <- stats.list$batch.ops
	num.batches <- length(batch.stepnums)
	node.color <- c()
	op.names <- c()
	config.colors <- c("tan", "darkseagreen", "dodgerblue", "lightyellow1", "khaki3", "indianred1", "grey18",  "goldenrod", "deeppink", "darkolivegreen", "antiquewhite", "greenyellow", "darkorange1", "blueviolet", "bisque2")
	for(b in 1:length(batch.stepnums)) {
		step.nums <- unlist(batch.stepnums[[b]])
		for(step.num in step.nums) {
			op.name <- GetOperatorId(step.num)
			op.name <- RenameOpId(op.name)
			op.name <- paste0(op.name,"[",step.num, "]")	
			op.names <- c(op.names, op.name)
			node.color<- c(node.color, config.colors[b])
		}
		
	}
	names(node.color) <- op.names
	graph.par(list(graph=list(main="Program DAG with Batch optimization[Batches are marked with different colors]", cex.main=0.8)))
	return(node.color)
}

GetBatchProgramDag <- function(pgm.name, dataset.id) {
	g <- GetProgramDag(pgm.name, dataset.id)
	node.list <- GenNodeList(dataset.id)
	batch.color <- GetColorCodingForBatches(node.list)
	nodeRenderInfo(g) <- list(fill=batch.color)
	return(g)
	
}
