
#Copyright 2013 Pradap  and Arun Kumar and Christopher Re
#
#Licensed under the Apache License, Version 2.0 (the "License");
#you may not use this file except in compliance with the License.
#You may obtain a copy of the License at
#
#http://www.apache.org/licenses/LICENSE-2.0
#
#Unless required by applicable law or agreed to in writing, software
#distributed under the License is distributed on an "AS IS" BASIS,
#WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#See the License for the specific language governing permissions and
#limitations under the License.
#
#
SetUpForEvalIgdTrain <- function(step.num) {
	res <- SetUpForIgdCoeffLearner(step.num)
	return(res)
}
SetUpForEvalCgTrain <- function(step.num) {
	res <- SetUpForCgCoeffLearner(step.num)
}

SetUpForEvalCvTest <- function(step.num) {
	
	feat.id <- GetInputSetId.1(step.num)
	set.val <- GetFeatureSet(feat.id)
	is.cg <- FALSE
	config <- GetIgdConfig(step.num, exit.flag = FALSE)
	if((is.list(config) == FALSE) && (config == MNULL)) {
		config <- GetCgConfig(step.num)
		is.cg <- TRUE
	}
	
	num.folds <- config$num.folds
	inp.list.id <- GetInputList.1(step.num)
	
	if(inp.list.id == MNULL) {
		init.wt <- config$init.wt
		len.set.val <- length(unlist(set.val))
		values <- rep(init.wt, len.set.val)
		coeff.list <- list()
		for(i in 1:num.folds) {
			coeff.list[[1]] <- list()
			coeff.list[[1]][[i]] <- values
		}
	} else {
		values <- GetFeatureMapValues(inp.list.id)
		values <- values[[1]]
		coeff.list <- values
	}
	res <- InitEvalCvTest(num.folds, set.val, coeff.list, is.cg)
	return(res)
}

SetUpForEvalAiCTest <- function(step.num) {
	
	feat.id <- GetInputSetId.1(step.num)
	set.val <- GetFeatureSet(feat.id)
	is.cg <- FALSE
	config <- GetIgdConfig(step.num, exit.flag = FALSE)
	if((is.list(config) == FALSE) && (config == MNULL)) {
		config <- GetCgConfig(step.num)
		is.cg <- TRUE
	}
	
	num.folds <- config$num.folds
	inp.list.id <- GetInputList.1(step.num)
	
	if(inp.list.id == MNULL) {
		init.wt <- config$init.wt
		len.set.val <- length(unlist(set.val))
		values <- rep(init.wt, len.set.val)
		coeff.list <- list()
		for(i in 1:num.folds) {
			coeff.list[[i]] <- list()
			coeff.list[[1]][[i]] <- values
		}
	} else {
		values <- GetFeatureMapValues(inp.list.id)
		values <- values[[1]]
	}
	res <- InitEvalAiCTest(num.folds, set.val, values, is.cg)
	return(res)
}

InitEvalCvTest <- function(num.folds, feat.set.list, values, cg.flag) {
	if(cg.flag == TRUE) {
		coeff.list <- values$coeff.list
		u.list <- values$u.list
		g.list <- values$g.list
	} else {
		coeff.list <- values
	}
	param.vec <- c(num.folds)
	res <- InitIgdCoeffLearner(param.vec, feat.set.list, coeff.list)
	return(res)
	
}

InitEvalAiCTest <- function(num.folds, feat.set.list, values, cg.flag) {
	
	if(cg.flag == TRUE) { 
		param.vec <- c(num.folds)
		coeff.list <- values$coeff.list
		u.list <- values$u.list
		g.list <- values$g.list
	}
	else {
		coeff.list <- values
		param.vec <- c(0, 0, num.folds)
	}
	res <- InitIgdCoeffLearner(param.vec, feat.set.list, coeff.list)
	return(res)
}


ResEvaluateAiCTest <- function(step.num, res.string) {
	exe.list <- SetUpForEvalAiCTest(step.num)
	len.param.vec <- length(exe.list$param.vec)

	if(len.param.vec == 3)
		num.folds <- exe.list$param.vec[3]
	else
		num.folds <- exe.list$param.vec[1]

	list.id <- GetOutputListId(step.num)
	res.list <- ParseIgdCoeffLearner(res.string, num.folds)
	SetFeatMapFeatSetList(list.id, res.list$feat.set.list)
	SetFeatureMapValues(list.id, res.list$coeff.list)
}


ResEvaluateIgdTrain <- function(step.num,  res.string) {
	ResIgdCoeffLearner(step.num, res.string)
}

ResEvaluateCgTrain <- function(step.num, res.string) {
	ResCgCoeffLearner(step.num, res.string)	
}

ResEvaluateCvTest <- function(step.num, res.string) {
	exe.list <- SetUpForEvalCvTest(step.num)
	num.folds <- exe.list$param.vec[1]

	list.id <- GetOutputListId(step.num)
	res.list <- ParseIgdCoeffLearner(res.string, num.folds)
	SetFeatMapFeatSetList(list.id, res.list$feat.set.list)
	coeff.list <- lapply(lapply(res.list$coeff.list, unlist), mean)
	SetFeatureMapValues(list.id, coeff.list)
}
