
#Copyright 2013 Pradap  and Arun Kumar and Christopher Re
#
#Licensed under the Apache License, Version 2.0 (the "License");
#you may not use this file except in compliance with the License.
#You may obtain a copy of the License at
#
#http://www.apache.org/licenses/LICENSE-2.0
#
#Unless required by applicable law or agreed to in writing, software
#distributed under the License is distributed on an "AS IS" BASIS,
#WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#See the License for the specific language governing permissions and
#limitations under the License.
#
#

# Script that contains the initialisations

# Load the libraries
library(data.table)
library(hash)

# Source the class definitions
source('classdefs.R')

# Source the helper functions
source('helperfuncs.R')

# Source the mdb utility functions
source('mdbutilities.R')

# Source the datalayer functions
source('datalayer_delegator.R')
source('datalayer_oracledb.R')



# Basic initializations to do a kick start 

# Basic infrastructure
Operators <- data.table()
Program <- data.table()
FeatureSetDB <- data.table()
FeatureMapDB <- data.table()
CompatiblePrograms <- data.table()

# Data.tables for data sets.
Dataset <- data.table()
Features <- data.table()
DerivedDatasets <- data.table()
DerivedFeatures <- data.table()

# Config tables
CgConfig <- data.table()
IgdConfig <- data.table()
BestKConfig <- data.table()


mdb = hash()
.set(mdb, Operators=Operators, Program=Program, FeatureSetDB=FeatureSetDB, FeatureMapDB=FeatureMapDB)
.set(mdb, CompatiblePrograms = CompatiblePrograms)
.set(mdb, Dataset=Dataset, DerivedDatasets=DerivedDatasets, Features=Features, DerivedFeatures = DerivedFeatures)
.set(mdb, CgConfig=CgConfig, IgdConfig=IgdConfig, BestKConfig=BestKConfig) 

mdb$program.lines <- list()
mdb$program.name <- 'ColumbusProgram'
mdb$session.id <- 0
mdb$program.id <- 1

	

# Update the Operator table; 
# TODO(pradap): Can be read from a config file.

# Operators provide the mapping between the operator and the data.table that it should refer to.
# The following information will be used while we have to store the provenance information.

Operators <- InsertRowInTable('Operators', data.table('AssignFeatureSet', 'FEATURE_SET', 'FeatureSetDB'))

Operators <- InsertRowInTable('Operators', data.table('FeatureMap', 'FEATURE_MAP', 'FeatureMapDB'))
Operators <- InsertRowInTable('Operators', data.table('AddFeatureSet', 'ADD_FEATURE', 'ManualOperationsDB'))
Operators <- InsertRowInTable('Operators', data.table('DeleteFeatureSet', 'DEL_FEATURE', 'ManualOperationsDB'))
Operators <- InsertRowInTable('Operators', data.table('FeatureSetIntersect', 'INTERSECT_FEATURE', 'ManualOperationsDB'))
Operators <- InsertRowInTable('Operators', data.table('BestK', 'BEST_K',  'BestKDB'))
Operators <- InsertRowInTable('Operators', data.table('CreateFeature', 'CREATE_FEATURE', 'DerivedFeaturesDB'))

# Descriptive statistics
Operators <- InsertRowInTable('Operators', data.table('CorrelationX', 'CORRELATION_X', 'CorrelationXDB'))
Operators <- InsertRowInTable('Operators', data.table('CorrelationY', 'CORRELATION_Y', 'CorrelationYDB'))
Operators <- InsertRowInTable('Operators', data.table('CoefficientLearner', 'COEF_TRAIN', 'CoefficientLearnerDB'))
Operators <- InsertRowInTable('Operators', data.table('Evaluate', 'EVALUATE', 'EvaluateDB'))

# Exploratory Operations
Operators <- InsertRowInTable('Operators', data.table('StepAdd', 'STEP_ADD', 'StepAddDB'))
Operators <- InsertRowInTable('Operators', data.table('StepDel', 'STEP_DEL', 'StepDelDB'))


# Define constants for the op.ids [ mapping from variable to op.id ]
FEATURE_SET <- 'FEATURE_SET'
FEATURE_MAP <- 'FEATURE_MAP'
ADD_FEATURE <- 'ADD_FEATURE'
DEL_FEATURE <- 'DEL_FEATURE'
INTERSECT_FEATURE <- 'INTERSECT_FEATURE'

BEST_K <- 'BEST_K'
CREATE_FEATURE <- 'CREATE_FEATURE'
CORRELATION_X <- 'CORRELATION_X'
CORRELATION_Y <- 'CORRELATION_Y'
COEF_IGD <- 'COEF_IGD_TRAIN'
COEF_CG <- 'COEF_CG_TRAIN'
EVALUATE <- 'EVALUATE'
STEP_ADD <- 'STEP_ADD'
STEP_DEL <- 'STEP_DEL'
EVAL_CG_TRAIN <- 'EVAL_CG_TRAIN'
EVAL_IGD_TRAIN <- 'EVAL_IGD_TRAIN'
EVAL_CV_TEST <- 'EVAL_CV_TEST'
EVAL_AIC_TEST <- 'EVAL_AIC_TEST'

STEPADD_EVAL_CG_TRAIN <- 'STEPADD_EVAL_CG_TRAIN'
STEPADD_EVAL_IGD_TRAIN <- 'STEPADD_EVAL_IGD_TRAIN'
STEPADD_EVAL_CV_TEST <- 'STEPADD_EVAL_CV_TEST'
STEPADD_EVAL_AIC_TEST <- 'STEPADD_EVAL_AIC_TEST'
STEPADD_BEST_K <- 'STEPADD_BEST_K'

STEPDEL_EVAL_CG_TRAIN <- 'STEPDEL_EVAL_CG_TRAIN'
STEPDEL_EVAL_IGD_TRAIN <- 'STEPDEL_EVAL_IGD_TRAIN'
STEPDEL_EVAL_CV_TEST <- 'STEPDEL_EVAL_CV_TEST'
STEPDEL_EVAL_AIC_TEST <- 'STEPDEL_EVAL_AIC_TEST'
STEPDEL_BEST_K <- 'STEPDEL_BEST_K'



# Define constants that we regularly use.
MNULL <- -1
COMPLETED <- 'COMPLETED'
LEVEL_PENDING <- -2

stats.list <- list()
RegisterAllDatasets()
source('managerial_user.R')
mdb$interactive <- TRUE

# Add a dataset
#Dataset <- InsertRowInTable('Dataset', data.table('ColumbusFordSmall', 31, 31))
#DerivedDatasets <- InsertRowInTable('DerivedDatasets', data.table('ColumbusFordSmall', 31, 31))

#RegisterDataset('ColumbusFordSmall', 31, 30)
#InitStats()
#r-studio

Sys.setenv(ORACLE_HOSTNAME="localhost.localdomain")
Sys.setenv(ORACLE_BASE="/u01/app/oracle")
Sys.setenv(ORACLE_HOME="/u01/app/oracle/product/11.2.0/dbhome_1")
Sys.setenv(ORACLE_SID="AL32UTF8")
Sys.setenv(NLS_LANG=".AL32UTF8")
Sys.setenv(ORACLE_UNQNAME = "AL32UTF8")
