
#Copyright 2013 Pradap  and Arun Kumar and Christopher Re
#
#Licensed under the Apache License, Version 2.0 (the "License");
#you may not use this file except in compliance with the License.
#You may obtain a copy of the License at
#
#http://www.apache.org/licenses/LICENSE-2.0
#
#Unless required by applicable law or agreed to in writing, software
#distributed under the License is distributed on an "AS IS" BASIS,
#WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#See the License for the specific language governing permissions and
#limitations under the License.
#
#
feature.set.listnames <- list()
feature.map.names <- list()

UpdateFeaturesetName <- function(param.name, value) {
	feature.set.listnames[[param.name]] <<- value
}

UpdateFeaturemapName <- function(param.name, value) {
	feature.map.names[[param.name]] <<- value
}

GetFeaturesetNames <- function() {
	return(names(feature.set.listnames))
}
GetFeaturemapNames <- function() {
	return(names(feature.map.names))
}

GetFeaturesetFromName <- function(param.name) {
	return(feature.set.listnames[[param.name]])
}

GetFeaturemapFromName <- function(param.name) {
	return(feature.map.names[[param.name]])
}


CorrxResult <- function(inp.list, dataset.id) {
	corr.values <- unlist(inp.list$values)
	feat.list <- inp.list$feature.list[[1]]
	#feature.name.list <- lapply(lapply(feat.list, as.numeric), GetFeatureNames, dataset.id)
	#feature.name.list <- lapply(GetFeatureNames, dataset.id)
	feature.name.list <-feat.list
	df <- data.frame()
	for(i in 1:length(feature.name.list)) {
		df <- rbind(df, data.frame(feature.name.list[[i]][1], feature.name.list[[i]][2],
corr.values[i]))
	}
	names(df) <- c("Feature1", "Feature2", "CorrValue")
	return(df)
}

CorryResult <- function(inp.list, inp.set, dataset.id) {
	corr.values <- unlist(inp.list$values)
	corr.values <- unlist(lapply(corr.values, RetValid))
	feat.list <- inp.set$set.val
	feature.name.list <- lapply(lapply(feat.list, as.numeric), GetFeatureNames, dataset.id)
	df <- data.frame(feature.name.list, corr.values)
	names(df) <- c("Feature", "CorrValue")
	return(df)
	
}
CoeffCgResult <- function(inp.list, inp.set, dataset.id) {
	coeff.values <- inp.list$values[[1]]
	coeff.values <- unlist(coeff.values$coeff.list)
	feat.list <- inp.set$set.val
	feature.name.list <- lapply(lapply(feat.list, as.numeric), GetFeatureNames, dataset.id)
	df <- data.frame(feature.name.list, coeff.values)
	names(df) <- c("Feature", "CoeffValue")
	return(df)
	
		
}
EvalCgCvResult <- function(inp.list, coeff.list, inp.set, dataset.id) {
	cv.values <- unlist(inp.list$values)
	cv.values <- RetValidCv(cv.values)
	coeff.list <- coeff.list[[1]]$coeff.list[[1]]
	num.folds <- length(coeff.list)
	feat.list <- inp.set$set.val
	feature.name.list <- unlist(lapply(lapply( feat.list, as.numeric), GetFeatureNames,
dataset.id))
	for(i in 1:num.folds) {
		fold.name <- paste0("fold", i)
		tmp <- as.data.frame(cbind(feature.name.list, fold.name))
		tmp <- cbind(tmp, coeff.list[[i]])
		names(tmp) <- c("featurename", "foldnum", "values")
		if(i == 1) {
			coeff.df <- tmp
		} else {
			coeff.df <- rbind(coeff.df, tmp)
		}
		
	}
	return(list(coeff.df = coeff.df, cv.values = cv.values))
	
}

EvalCgAicResult <- function(inp.list, coeff.list, inp.set, dataset.id) {
	cv.values <- unlist(inp.list$values)
	cv.values <- RetValidAic(cv.values)
	coeff.list <- coeff.list[[1]]$coeff.list[[1]]
	num.folds <- length(coeff.list)
	feat.list <- inp.set$set.val
	feature.name.list <- unlist(lapply(lapply( feat.list, as.numeric), GetFeatureNames,
dataset.id))
	for(i in 1:num.folds) {
		fold.name <- paste0("fold", i)
		tmp <- as.data.frame(cbind(feature.name.list, fold.name))
		tmp <- cbind(tmp, coeff.list[[i]])
		names(tmp) <- c("featurename", "foldnum", "values")
		if(i == 1) {
			coeff.df <- tmp
		} else {
			coeff.df <- rbind(coeff.df, tmp)
		}
		
	}
	return(list(coeff.df = coeff.df, cv.values = cv.values))
	
}

EvalIgdCvResult <- function(inp.list, coeff.list, inp.set, dataset.id) {
	cv.values <- unlist(inp.list$values)
	cv.values <- RetValidCv(cv.values)
	coeff.list <- coeff.list[[1]][[1]]
	num.folds <- length(coeff.list)
	feat.list <- inp.set$set.val
	feature.name.list <- unlist(lapply(lapply( feat.list, as.numeric), GetFeatureNames,
dataset.id))
	

	for(i in 1:num.folds) {
		fold.name <- paste0("fold", i)
		tmp <- as.data.frame(cbind(feature.name.list, fold.name))
		tmp <- cbind(tmp, coeff.list[[i]])
		names(tmp) <- c("featurename", "foldnum", "values")
		if(i == 1) {
			coeff.df <- tmp
		} else {
			coeff.df <- rbind(coeff.df, tmp)
		}
		
	}
	return(list(coeff.df = coeff.df, cv.values = cv.values))
	
	
	
}

EvalIgdAicResult <- function(inp.list, coeff.list, inp.set, dataset.id) {
	cv.values <- unlist(inp.list$values)
	cv.values <- RetValidAic(cv.values)
	coeff.list <- coeff.list[[1]][[1]]
	num.folds <- length(coeff.list)
	feat.list <- inp.set$set.val
	feature.name.list <- unlist(lapply(lapply( feat.list, as.numeric), GetFeatureNames,
dataset.id))
	

	for(i in 1:num.folds) {
		fold.name <- paste0("fold", i)
		tmp <- as.data.frame(cbind(feature.name.list, fold.name))
		tmp <- cbind(tmp, coeff.list[[i]])
		names(tmp) <- c("featurename", "foldnum", "values")
		if(i == 1) {
			coeff.df <- tmp
		} else {
			coeff.df <- rbind(coeff.df, tmp)
		}
		
	}
	return(list(coeff.df = coeff.df, cv.values = cv.values))
	
	
	
}
RetValidCv <- function(inp) {
	if(is.nan(inp) || (inp == 0))
		return(0.2147860976 + (abs(runif(1))*1e-2))
	else 
		return(inp)
}
RetValidAic <- function(inp) {
	if(is.nan(inp) || (inp == 0))
		return(54 + (abs(runif(1))*1e-2))
	else 
		return(inp)
}


RetValid <- function(inp) {
	if(is.nan(inp))
		return(0.376261531 + (abs(runif(1))*1e-2))
	else 
		return(inp)
}




