
#Copyright 2013 Pradap  and Arun Kumar and Christopher Re
#
#Licensed under the Apache License, Version 2.0 (the "License");
#you may not use this file except in compliance with the License.
#You may obtain a copy of the License at
#
#http://www.apache.org/licenses/LICENSE-2.0
#
#Unless required by applicable law or agreed to in writing, software
#distributed under the License is distributed on an "AS IS" BASIS,
#WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#See the License for the specific language governing permissions and
#limitations under the License.
#
#
SkelCorrelationX <- function(inp.set, dataset.id) {
	
	# Hard code the set.val to be NA, as it is just dummy at this point of time.
	set.val <- NA 
	feat.list <- list(set.val)
	values <- list(NA)
	
	step.num <- GetStepNum() + 1
	list.id <- InsertRowInTable('FeatureMapDB', data.table(step.num, feat.list, values))
	pgm.ctr <- step.num
	
	inpset.id2 <- MNULL
	inplist.id1 <- MNULL
	outset.id <- MNULL
	
	res.step.num <- InsertRowInTable('Program', data.table(dataset.id, CORRELATION_X,
				step.num, pgm.ctr, inp.set$feat.id, inpset.id2, inplist.id1, outset.id, list.id))
	
	out.list <- FeatureMapClass(list.id, feat.list, values)
	return(list(out.list = out.list, step.num = step.num))
}

SkelCorrelationY <- function(inp.set1, inp.set2, dataset.id) {
	
	set.val <- NA
	list.val <- NA

	feat.list <- list(set.val)
	values <- list(list.val)
	
	step.num <- GetStepNum() + 1
	list.id <- InsertRowInTable('FeatureMapDB', data.table(step.num, feat.list, values))
	pgm.ctr <- step.num

	inplist.id1 <- MNULL
	outset.id <- MNULL

	res.step.num <- InsertRowInTable('Program', data.table(dataset.id, CORRELATION_Y, step.num,
					pgm.ctr, inp.set1$feat.id, inp.set2$feat.id, inplist.id1, outset.id, list.id))
	out.list <- FeatureMapClass(list.id, feat.list, values)
	return(list(out.list = out.list, step.num = step.num))

}

SkelIgdCoeffLearner <- function(inp.set, num.iters, step.size, decay, init.wt, num.folds, dataset.id) {

	set.val <- inp.set$set.val
	values <- list(NA)

	step.num.list <- c()
	
	step.num <- GetStepNum() + 1
	pgm.ctr <- step.num # fixed.
	InsertRowInTable('IgdConfig', data.table(dataset.id, step.num, pgm.ctr, num.iters,
					step.size, decay, init.wt, num.folds))
	



	for(i in 1:num.iters) {

		list.id <- InsertRowInTable('FeatureMapDB', data.table(step.num, set.val, values))
		step.num.list <- c(step.num.list, step.num)
		inpset.id2 <- MNULL
		outset.id <- MNULL
		if(i == 1) 
			inplist.id <- MNULL
		else
			inplist.id <- old.list.id

		res.step.num <- InsertRowInTable('Program', data.table(dataset.id, COEF_IGD, step.num,
					pgm.ctr, inp.set$feat.id, inpset.id2, inplist.id, outset.id, list.id))
		step.num <- GetStepNum() + 1
		old.list.id <- list.id
	
	}

	out.list <- FeatureMapClass(list.id, set.val, values)
	return(list(out.list = out.list, step.num.list = step.num.list))
	
}

SkelIgdEvaluate <- function(inp.set, eval.type, num.iters, step.size, decay, init.wt, num.folds,
dataset.id) {
	
	set.val <- inp.set$set.val
	values <- list(NA)

	step.num.list <- c()
	
	step.num <- GetStepNum() + 1
	pgm.ctr <- step.num # fixed

	type <- tolower(eval.type)

	if(type == "aic") {
		EVAL_TEST <- EVAL_AIC_TEST
		if(num.folds != 1) {	
			stop('Argument num.folds must be equal to 1 for AIC')
		}
	} else if(type == "cv") {
		EVAL_TEST <- EVAL_CV_TEST
	} else {
		stop('Unknown eval type')
	}

	InsertRowInTable('IgdConfig', data.table(dataset.id, step.num, pgm.ctr, num.iters,
step.size, decay, init.wt, num.folds))

	# Train Phase
	
	for(i in 1:num.iters) {
		list.id <- InsertRowInTable('FeatureMapDB', data.table(step.num, set.val, values))
		step.num.list <- c(step.num.list, step.num)
		inpset.id2 <- MNULL
		outset.id <- MNULL

		if(i == 1)
			inplist.id <- MNULL
		else
			inplist.id <- old.list.id
		
		res.step.num <- InsertRowInTable('Program', data.table(dataset.id, EVAL_IGD_TRAIN,
step.num, pgm.ctr, inp.set$feat.id, inpset.id2, inplist.id, outset.id, list.id))
		
		step.num <- GetStepNum() + 1
		old.list.id <- list.id
	}

	# Test Phase
	step.num <- GetStepNum() + 1
	step.num.list <- c(step.num.list, step.num)
	list.id <- InsertRowInTable('FeatureMapDB', data.table(step.num, set.val, values))

	inpset.id2 <- MNULL
	outset.id <- MNULL
	
	res.step.num <- InsertRowInTable('Program', data.table(dataset.id, EVAL_TEST, step.num,
pgm.ctr, inp.set$feat.id, inpset.id2, old.list.id, outset.id, list.id))
	
	out.list <- FeatureMapClass(list.id, set.val, values)
	return(list(out.list = out.list, step.num.list = step.num.list))
	
}

SkelCgEvaluate <- function(inp.set, eval.type, num.iters, init.wt, num.folds, dataset.id) {
	
	set.val <- inp.set$set.val
	values <- list(NA)
	
	step.num.list <- c()
	step.num <- GetStepNum() + 1
	pgm.ctr <- step.num # fixed !!

	type <- tolower(eval.type)
	
	if(type == "aic") {
		EVAL_TEST <- EVAL_AIC_TEST
		if(num.folds != 1) {
			stop('Argument num.folds must be equal to 1 for AIC')
		}
	} else if(type == "cv") {
		EVAL_TEST <- EVAL_CV_TEST
	} else {
		stop('Unknown eval type')
	}

	InsertRowInTable('CgConfig', data.table(dataset.id, step.num, pgm.ctr, num.iters, init.wt,
num.folds))
	
	for(i in 1:num.iters) {
		list.id <- InsertRowInTable('FeatureMapDB', data.table(step.num, set.val, values))
		step.num.list <- c(step.num.list, step.num)
		inpset.id2 <- MNULL
		outset.id <- MNULL
	
		if(i == 1)
			inplist.id <- MNULL
		else
			inplist.id <- old.list.id
		
		res.step.num <- InsertRowInTable('Program', data.table(dataset.id, EVAL_CG_TRAIN, step.num,
pgm.ctr, inp.set$feat.id, inpset.id2, inplist.id, outset.id, list.id))
		
		step.num <- GetStepNum() + 1
		old.list.id <- list.id
		
	}	

	# Test Phase
	step.num <- GetStepNum() + 1
	step.num.list <- c(step.num.list, step.num)
	list.id <- InsertRowInTable('FeatureMapDB', data.table(step.num, set.val, values))
	inpset.id2 <- MNULL
	outset.id <- MNULL

	res.step.num <- InsertRowInTable('Program', data.table(dataset.id, EVAL_TEST, step.num,
pgm.ctr, inp.set$feat.id, inpset.id2, old.list.id, outset.id, list.id))
	
	out.list <- FeatureMapClass(list.id, set.val, values)
	return(list(out.list = out.list, step.num.list = step.num.list))
		
	
	
}


SkelCgCoeffLearner <- function(inp.set, num.iters, init.wt, num.folds, dataset.id) {

	set.val <- inp.set$set.val
	values <- list(NA)

	step.num.list <- c()

	step.num <- GetStepNum() + 1
	pgm.ctr <- step.num # fixed
	
	InsertRowInTable('CgConfig', data.table(dataset.id, step.num, pgm.ctr, num.iters, init.wt,
					num.folds))
	
	for(i in 1:num.iters) {
		
		list.id <- InsertRowInTable('FeatureMapDB', data.table(step.num, set.val, values))
		step.num.list <- c(step.num.list, step.num)
		inpset.id2 <- MNULL
		outset.id <- MNULL
		
		if(i == 1)
			inplist.id <- MNULL
		else
			inplist.id <- old.list.id

		res.step.num <- InsertRowInTable('Program', data.table(dataset.id, COEF_CG,
				step.num, pgm.ctr, inp.set$feat.id, inpset.id2, inplist.id, outset.id, list.id))
		step.num <- GetStepNum() + 1
		old.list.id <- list.id
		
	}

	out.list <- FeatureMapClass(list.id, set.val, values)
	return(list(out.list = out.list, step.num.list = step.num.list))
}
