
#Copyright 2013 Pradap  and Arun Kumar and Christopher Re
#
#Licensed under the Apache License, Version 2.0 (the "License");
#you may not use this file except in compliance with the License.
#You may obtain a copy of the License at
#
#http://www.apache.org/licenses/LICENSE-2.0
#
#Unless required by applicable law or agreed to in writing, software
#distributed under the License is distributed on an "AS IS" BASIS,
#WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#See the License for the specific language governing permissions and
#limitations under the License.
#
#
# Skeleton for Add feature set routine. The function populates the program, feature set/map tables
# with dummy values
#
# Args:
#  inp.set1 : One of input feature set of type feature set class.
#  inp.set2 : Anther input feature set of type feature set class.
#  dataset.id : Dataset id to which the feature sets belong to.
#
# Returns:
#   List containing step.num and out.set
#   step.num : The step number in the program table.
#   out.set  : Dummy output of type feature set class. The set val is filled with NA

SkelAddFeatureSet <- function(inp.set1, inp.set2, dataset.id) {
	
	step.num <- GetStepNum() + 1
	id1 <- inp.set1$feat.id
	id2 <- inp.set2$feat.id

	set.val <- NA
	
	# Insert into feature set db
	feat.id <- InsertRowInTable('FeatureSetDB', data.table(step.num, list(set.val)))
	pgm.ctr <- step.num
	inplist.id1 <- MNULL
	outlist.id <- MNULL
	
	res.step.num <- InsertRowInTable('Program', data.table(dataset.id, ADD_FEATURE, step.num, 
					pgm.ctr, id1, id2, inplist.id1, feat.id, outlist.id))
	out.set <- FeatureSetClass(feat.id, set.val)
	return(list(step.num = step.num, out.set = out.set))
	
}
# Skeleton for Del feature set routine. The function populates the program, feature set/map tables
# with dummy values
#
# Args:
#  inp.set1 : One of input feature set of type feature set class.
#  inp.set2 : Anther input feature set of type feature set class.
#  dataset.id : Dataset id to which the feature sets belong to.
#
# Returns:
#   List containing step.num and out.set
#   step.num : The step number in the program table.
#   out.set  : Dummy output of type feature set class. The set val is filled with NA


SkelDelFeatureSet <- function(inp.set1, inp.set2, dataset.id) {
	
	step.num <- GetStepNum() + 1
	id1 <- inp.set1$feat.id
	id2 <- inp.set2$feat.id

	set.val <- NA
	
	# Insert into feature set db
	feat.id <- InsertRowInTable('FeatureSetDB', data.table(step.num, list(set.val)))
	pgm.ctr <- step.num
	inplist.id1 <- MNULL
	outlist.id <- MNULL
	
	res.step.num <- InsertRowInTable('Program', data.table(dataset.id, DEL_FEATURE, step.num, 
					pgm.ctr, id1, id2, inplist.id1, feat.id, outlist.id))
	out.set <- FeatureSetClass(feat.id, set.val)
	return(list(step.num = step.num, out.set = out.set))
	
}

# Skeleton for intersect feature set routine. The function populates the program, feature set/map tables
# with dummy values
#
# Args:
#  inp.set1 : One of input feature set of type feature set class.
#  inp.set2 : Anther input feature set of type feature set class.
#  dataset.id : Dataset id to which the feature sets belong to.
#
# Returns:
#   List containing step.num and out.set
#   step.num : The step number in the program table.
#   out.set  : Dummy output of type feature set class. The set val is filled with NA

SkelIntersectFeatureSet <- function(inp.set1, inp.set2, dataset.id) {
	
	step.num <- GetStepNum() + 1
	id1 <- inp.set1$feat.id
	id2 <- inp.set2$feat.id

	set.val <- NA
	
	# Insert into feature set db
	feat.id <- InsertRowInTable('FeatureSetDB', data.table(step.num, list(set.val)))
	pgm.ctr <- step.num
	inplist.id1 <- MNULL
	outlist.id <- MNULL
	
	res.step.num <- InsertRowInTable('Program', data.table(dataset.id, INTERSECT_FEATURE, step.num, 
					pgm.ctr, id1, id2, inplist.id1, feat.id, outlist.id))
	out.set <- FeatureSetClass(feat.id, set.val)
	return(list(step.num = step.num, out.set = out.set))
	
}


SkelBestK <- function(inp.list, k.val, dataset.id) {
	
	step.num <- GetStepNum() + 1
	list.id <- inp.list$list.id
	set.val <- NA
	
	# Insert into feature set db
	feat.id <- InsertRowInTable('FeatureSetDB', data.table(step.num, list(set.val)))
	pgm.ctr <- step.num
	inpset.id1 <- MNULL
	inpset.id2 <- MNULL
	outlist.id <- MNULL

	res.step.num <- InsertRowInTable('Program', data.table(dataset.id, BEST_K, step.num,
					pgm.ctr, inpset.id1, inpset.id2, list.id, feat.id, outlist.id))

	# Insert into BestKConfig 
	InsertRowInTable('BestKConfig', data.table(dataset.id, step.num, pgm.ctr, k.val))
	
	out.set <- FeatureSetClass(feat.id, set.val)
	return(list(step.num = step.num, out.set = out.set))
	 
}
