DROP TABLE StockTable;
CREATE TABLE StockTable (
  ticker VARCHAR(4),
  open_price NUMBER,
  close_price NUMBER
)
/

-- Define the ref cursor type
DROP PACKAGE refcur_pkg;
CREATE PACKAGE refcur_pkg IS
  TYPE refcur_t IS REF CURSOR RETURN StockTable%ROWTYPE;
END refcur_pkg;
/

-- Define object to hold values from each column
CREATE or REPLACE TYPE TickerType01 AS OBJECT 
(
 price NUMBER
);

CREATE OR REPLACE TYPE TickerTypeSet01 AS TABLE OF TickerType01;
/

CREATE or REPLACE TYPE StockPivotImpl AS OBJECT
(
  key INTEGER,

  STATIC FUNCTION ODCITableStart(sctx OUT StockPivotImpl, cur SYS_REFCURSOR)
    RETURN NUMBER
    AS LANGUAGE JAVA
    NAME 'StockPivotImpl.ODCITableStart(oracle.sql.STRUCT[], java.sql.ResultSet) return java.math.BigDecimal',

  MEMBER FUNCTION ODCITableFetch(self IN OUT StockPivotImpl, nrows IN NUMBER,
                                 outSet OUT TickerTypeSet01) RETURN NUMBER
    AS LANGUAGE JAVA
    NAME 'StockPivotImpl.ODCITableFetch(java.math.BigDecimal, oracle.sql.ARRAY[]) return java.math.BigDecimal',

  MEMBER FUNCTION ODCITableClose(self IN StockPivotImpl) RETURN NUMBER
    AS LANGUAGE JAVA
    NAME 'StockPivotImpl.ODCITableClose() return java.math.BigDecimal'

);
/
show errors
/

-- Create function to retrieve the numbers.
CREATE or REPLACE FUNCTION StockPivot(p refcur_pkg.refcur_t) RETURN TickerTypeSet01
PIPELINED USING StockPivotImpl;
/
set serveroutput on;
set timing on;

DROP type oreT;
CREATE OR REPLACE type oreT as object
(
	ctr number,
	static function initFeatureLengths(val IN VARCHAR2) return number as language java name 'MyUDFManager.initNumAttributes(java.lang.String) return int',
	static function initDecodeIndices(val in VARCHAR2) return NUMBER as language java name 'MyUDFManager.initDecodeIndices(java.lang.String) return int',
	static function initDecodeIndicesMap(val in VARCHAR2) return NUMBER as language java name 'MyUDFManager.initDecodeIndicesMap(java.lang.String) return int',
	static function setYIdx(val in VARCHAR2) return NUMBER as language java name 'MyUDFManager.setYIdx(java.lang.String) return int',
	static function initialize(val in VARCHAR2) return number as language java name 'MyUDFManager.initialize(java.lang.String) return int',
	static function initializeInStages(val in VARCHAR2) return number as language java name 'MyUDFManager.initializeInStages(java.lang.String) return int',
	static function initObjectList(val in VARCHAR2) return number as language java name 'MyUDFManager.initObjectList(java.lang.String) return int',
	static function initNewFeatures(val in VARCHAR2) return number as language java name 'MyUDFManager.initNewFeatures(java.lang.String) return int',
	static function iterate(val in RAW) return NUMBER as language java name 'MyUDFManager.iterate(byte[]) return int',
	static function result(val in VARCHAR2) return VARCHAR2 as language java name 'MyUDFManager.result(java.lang.String) return String',
	static function getResultInStages(val in VARCHAR2) return VARCHAR2 as language java name 'MyUDFManager.getResultInStages(java.lang.String) return String',
	static function terminate return number as language java name 'MyUDFManager.terminate() return int',
	static function nullUDF(val in raw) return number as language java name 'MyUDFManager.nullUDF(byte[]) return int',
	static function noDataFunction return number as language java name 'MyUDFManager.noDataFunction() return int',
	static function nullAndParse(val in raw) return number as language java name 'MyUDFManager.nullAndParse(byte[]) return int',
	static function materialize(val in raw) return raw as language java name 'MyUDFManager.materialize(byte[]) return byte[]',
	static function initMatSize(val in VARCHAR2) return number as language java name 'MyUDFManager.initMatSize(java.lang.String) return int',
	static function createObj(val in VARCHAR2) return VARCHAR2 as language java name 'MyUDFManager.createObj(java.lang.String) return java.lang.String',
	static function deleteObj(val in VARCHAR2) return number as language java name 'MyUDFManager.deleteObj(java.lang.String) return int'
);
/
