
public class MyEvalLRCVTrain extends MyAggregate{
    private MyFeatureSubsetLRCV objFS[];

    private double initStep;
    private double decay;
    private int numFolds;

    public int initialize(String s) {
        // should initialize feature subsets and the coef values.
        // schema initparams#fs1#fs2#fs3
        // initparams : init step, decay, numFolds


        String []splitted = s.split("#");

        String[] paramSplitted = splitted[0].split(",");
        //initialize parameters


        initStep =  Double.parseDouble(paramSplitted[0]);
        decay = Double.parseDouble(paramSplitted[1]);
        numFolds = Integer.parseInt(paramSplitted[2]);

        objFS = new MyFeatureSubsetLRCV[splitted.length - 1]; //because the first one was the fold num.
        for(int i = 1; i < splitted.length; i ++) {
            objFS[i-1]= new MyFeatureSubsetLRCV();
            objFS[i-1].initFeatureSubset(numFolds, splitted[i]);
        }
        return 1;
    }
    private int train(int foldNum) {
        for(int i = 0; i < objFS.length; i ++) {
            int numFeatures = objFS[i].featureIndices.length;

            double wx = 0.0;
            for(int j = 0; j < numFeatures; j ++ ) {
                int featureIndex = objFS[i].featureIndices[j];
                double wVal = objFS[i].coefficients[foldNum][j]; //fold is hardcoded
                double x = MyTuple.xVal[featureIndex];
                wx += wVal*x;
            }
            double y = MyTuple.yVal;
	    if(y == 0.0) 
		y = 0.4;
	    else if(y == 1.0)
		y = 0.6;

            double v = (-1*wx*y)/1e5;
            double sig;
            if(v > 30)
            {
                sig = (1.0/1.0+Math.exp(-1*v));
            }
            else
            {
                double t =  Math.exp(-1*v);
                sig = t/(1+t);
            }
	    //SimpleLogger.write("Y : " + y + "; V : " + v + "; sig = " + sig); 
            double stp = sig*y*initStep*decay;
            for(int j = 0; j < numFeatures; j ++ ) {
                int featureIndex = objFS[i].featureIndices[j];
                double wVal = objFS[i].coefficients[foldNum][j]; //fold is hardcoded
                double x = MyTuple.xVal[featureIndex];
                double wvn = wVal + stp * x; 
                objFS[i].coefficients[foldNum][j] = wvn;
                //objFS[i].coefficients[foldNum][j] = Math.random();
            }

        }
        return 1;
    }
    public int iterate() {
	if(numFolds == 1) {
		train(0);
	}
        else { 
		for(int i = 0; i < numFolds; i ++) {
            		if( ((MyTuple.recId % numFolds) + 1) != (i + 1)) {
		                train(i);
            		}
        	}
	}
        return 1;
    }
    public String result() {
        StringBuilder sb = new StringBuilder();
        String res;
        // layout : fs1 # fs2 # fs3
        //        : fs1 = featureSetIndices $ fold1 ~ fold2 ~ fold3 ~ ... ~ foldk
        //        : fold1 = coefficient values
        for(int i = 0; i < objFS.length; i ++)
        {
            res = objFS[i].getCoefResult();
            sb.append(res);
            if(i != objFS.length - 1)
                sb.append("#");
        }
        return sb.toString();
    }
    public int terminate() {
        objFS = null;
        return 1;
    }
}
